@@ -1,11 +1,11 @@
 import pickle
-from typing import Optional, List, Dict, Callable
+from typing import Optional, List, Dict, Callable, IO
 
 import ipywidgets as widgets
 import k3d
 import numpy as np
 from IPython.display import display
-from ipywidgets import Play
+from ipywidgets import Play, Checkbox
 from k3d.plot import Plot
 
 from .jointset import JointSet, MuPoTSJoints, OpenPoseJoints, CocoExJoints, PanopticJoints, Common14Joints
@@ -19,16 +19,6 @@ class FrameUpdater:
         self.joint_set: JointSet = joint_set
         self.video_player: Play = video_player
 
-    def update_plot(self, frame_index) -> None:
-        current_frame = self.frames[frame_index.new]
-        number_of_skeletons: int = len(self.skeletons)
-        for i in range(number_of_skeletons):
-            for j in range(len(self.joint_set.limb_graph)):
-                line_indices = self.joint_set.limb_graph[j]
-                self.skeletons[i].joint_lines[j].vertices = \
-                    (current_frame[i][line_indices[0]], current_frame[i][line_indices[1]])
-            self.skeletons[i].joint_points.positions = current_frame[i]
-
     def update_to_next_frame(self, button) -> None:
         if self.video_player.max > self.video_player.value:
             self.video_player.value += 1
@@ -44,13 +34,18 @@ class SkeletonVisualizer:
         self.plot: Optional[Plot] = None
         self.size_scalar: float = size_scalar
         self.skeletons: List[DrawableSkeleton] = []
-        self.joint_names_visible = widgets.Checkbox(description="Show Joint Names")
+        self.joint_names_visible: Checkbox = Checkbox(description="Show Joint Names")
+        self.joint_coordinates_visible: Checkbox = Checkbox(description="Show Coordinates")
 
-    def visualize(self, skeletons: np.ndarray, colors: List[Color] = None, include_names: bool = False) -> None:
+    def visualize(self, skeletons: np.ndarray, colors: List[Color] = None,
+                  include_names: bool = False, include_coordinates: bool = False) -> None:
         self.__assert_skeleton_shapes(skeletons)
         colors = self.__init_colors(skeletons.shape[0], colors)
+        self.__assert_include_arguments(include_names, include_coordinates)
         if include_names:
             skeleton_converter = Skeleton.to_drawable_skeleton_with_names
+        elif include_coordinates:
+            skeleton_converter = Skeleton.to_drawable_skeleton_with_coordinates
         else:
             skeleton_converter = Skeleton.to_drawable_skeleton
         self.__create_skeleton_plot(skeletons, skeleton_converter, colors)
@@ -58,54 +53,56 @@ class SkeletonVisualizer:
         if include_names:
             self.__link_joint_name_visibility_with_checkbox()
             display(self.joint_names_visible)
-        self.__display_color_changer()
 
-    def visualize_video_from_file(
-            self, file_name: str, colors: List[Color] = None,
-            fps: int = 15, include_names: bool = False) -> None:
-        file = open(file_name, 'rb')
+    def visualize_video_from_file(self, file_name: str, colors: List[Color] = None, fps: int = 15,
+                                  include_names: bool = False, include_coordinates: bool = False) -> None:
+        file: IO = open(file_name, 'rb')
         frames = pickle.load(file)
         file.close()
-        self.visualize_video(frames, colors, fps, include_names)
+        self.visualize_video(frames, colors, fps, include_names, include_coordinates)
 
-    def visualize_video(
-            self, frames: np.ndarray, colors: List[Color] = None,
-            fps: int = 15, include_names: bool = False) -> None:
+    def visualize_video(self, frames: np.ndarray, colors: List[Color] = None, fps: int = 15,
+                        include_names: bool = False, include_coordinates: bool = False) -> None:
         assert len(frames.shape) == 4
-        first_frame = frames[0]
+        first_frame: np.ndarray = frames[0]
         self.__assert_skeleton_shapes(first_frame)
         colors = self.__init_colors(first_frame.shape[0], colors)
         skeleton_timestamps: List[Dict[str, np.ndarray]] = self.__get_skeleton_positions_timestamps(frames)
+        self.__assert_include_arguments(include_names, include_coordinates)
+        if include_names:
+            skeleton_converter = None
+        elif include_coordinates:
+            skeleton_converter = Skeleton.to_drawable_skeleton_for_video_with_coordinates
+        else:
+            skeleton_converter = Skeleton.to_drawable_skeleton_for_video
         self.__create_skeleton_plot(
-            skeletons=first_frame,
-            skeleton_converter=Skeleton.to_drawable_skeleton_for_video,
+            skeletons=first_frame, skeleton_converter=skeleton_converter,
             colors=colors, positions=skeleton_timestamps)
         self.plot.display()
         self.__display_video_player(fps, frames)
 
     @staticmethod
     def __get_skeleton_positions_timestamps(frames: np.ndarray) -> List[Dict[str, np.ndarray]]:
-        frames_swapped = np.swapaxes(frames, 0, 1)
+        frames_swapped: np.ndarray = np.swapaxes(frames, 0, 1)
         return [
             {
                 str(timestamp_index): frames_swapped[current_frame_index][timestamp_index]
                 for timestamp_index in range(len(frames_swapped[current_frame_index]))
-            }
-            for current_frame_index in range(len(frames_swapped))
+            } for current_frame_index in range(len(frames_swapped))
         ]
 
     def __create_skeleton_plot(self, skeletons: np.ndarray, skeleton_converter: Callable[[Skeleton], DrawableSkeleton],
                                colors: List[Color], positions: Optional[Positions] = None) -> None:
         self.__init_skeleton_plot(skeletons)
-        skeleton_part_size = self.__calculate_skeleton_part_size(skeletons)
+        skeleton_part_size: float = self.__calculate_skeleton_part_size(skeletons)
         if positions is None:
             positions = skeletons
         skeletons: List[Skeleton] = self.__get_skeletons(positions, colors, skeleton_part_size)
         drawable_skeletons: List[DrawableSkeleton] = [skeleton_converter(skeleton) for skeleton in skeletons]
         self.__add_skeletons_to_plot(drawable_skeletons)
 
-    def __init_skeleton_plot(self, skeletons: np.ndarray):
-        self.plot = k3d.plot(camera_auto_fit=False)
+    def __init_skeleton_plot(self, skeletons: np.ndarray) -> None:
+        self.plot = k3d.plot(antialias=1, camera_auto_fit=False)
         centroid: np.ndarray = np.average(np.average(skeletons, axis=0), axis=0)
         camera_up_vector: np.ndarray = np.zeros(shape=(3,))
         for line_indices in self.joint_set.vertically_aligned_line_indices:
@@ -122,30 +119,29 @@ class SkeletonVisualizer:
         max_values = [abs(skeleton).max() for skeleton in skeletons]
         return (min(max_values) / 100.0) * self.size_scalar
 
-    def __get_skeletons(self, positions: List[Positions],
-                        colors: List[Color], skeleton_part_size: float) -> List[Skeleton]:
-        return [skeleton for skeleton in map(
-                lambda position_color_tuple: Skeleton(
-                    joint_positions=position_color_tuple[0],
-                    joint_set=self.joint_set,
-                    part_size=skeleton_part_size,
-                    color=position_color_tuple[1]),
-                zip(positions, colors))]
+    def __get_skeletons(self, positions: List[Positions], colors: List[Color],
+                        skeleton_part_size: float) -> List[Skeleton]:
+        return list(map(lambda position_color_tuple: Skeleton(
+            joint_positions=position_color_tuple[0],
+            joint_set=self.joint_set,
+            part_size=skeleton_part_size,
+            color=position_color_tuple[1]
+        ), zip(positions, colors)))
 
-    def __add_skeletons_to_plot(self, skeletons: List[DrawableSkeleton]):
+    def __add_skeletons_to_plot(self, skeletons: List[DrawableSkeleton]) -> None:
         for drawable_skeleton in skeletons:
             self.plot += drawable_skeleton
             self.skeletons.append(drawable_skeleton)
 
-    def __display_video_player(self, fps, frames):
+    def __display_video_player(self, fps, frames) -> None:
         video_player: Play = Play(
             value=0,
-            min=0, max=frames.shape[1] - 1,
+            min=0, max=frames.shape[0] - 1,
             step=1, interval=1000 / fps,
             description='Press play', disabled=False
         )
         frame_updater = FrameUpdater(self.skeletons, frames, self.joint_set, video_player)
-        frame_slider = widgets.IntSlider(value=0, min=0, max=frames.shape[1] - 1, step=1, description='Frame')
+        frame_slider = widgets.IntSlider(value=0, min=0, max=frames.shape[0] - 1, step=1, description='Frame')
         next_frame_button = widgets.Button(description='Next frame')
         next_frame_button.on_click(frame_updater.update_to_next_frame)
         previous_frame_button = widgets.Button(description='Prev frame')
@@ -165,18 +161,20 @@ class SkeletonVisualizer:
 
     def __assert_skeleton_shapes(self, skeletons: np.ndarray) -> None:
         assert len(skeletons.shape) == 3, 'The \'skeletons\' parameter should be a 3 dimensional numpy array.'
-        assert skeletons.shape[1] == self.joint_set.number_of_joints,\
+        assert skeletons.shape[1] == self.joint_set.number_of_joints, \
             'The number of joints of skeletons and the number of joints in the specified joint set must be the same.'
         assert skeletons.shape[2] == 3, 'The skeleton joint coordinates must be 3 dimensional'
 
+    @staticmethod
+    def __assert_include_arguments(include_names: bool, include_coordinates) -> None:
+        if include_names is True and include_coordinates is True:
+            raise AttributeError('Either names or coordinates can be showed, but not both.')
+
     def __link_joint_name_visibility_with_checkbox(self) -> None:
         for skeleton in self.skeletons:
             for joint_name in skeleton.joint_names:
                 widgets.jslink((joint_name, 'visible'), (self.joint_names_visible, 'value'))
 
-    def __display_color_changer(self):
-        pass
-
 
 class MuPoTSVisualizer(SkeletonVisualizer):
     def __init__(self, size_scalar: float = 1.0):
